@@ -1,6 +1,7 @@
 """test_generation"""
 
 import sys, time, unittest
+import urllib
 from routes import *
 
 class TestGeneration(unittest.TestCase):
@@ -454,6 +455,14 @@ class TestGeneration(unittest.TestCase):
         assert '/blog/content' == m.generate(controller='content')
         assert '/blog/admin/comments' == m.generate(controller='admin/comments')
 
+        m.environ = dict(SCRIPT_NAME='/notblog')
+
+        assert '/notblog/content/view' == m.generate(controller='content', action='view')
+        assert '/notblog/content' == m.generate(controller='content')
+        assert '/notblog/content' == m.generate(controller='content')
+        assert '/notblog/admin/comments' == m.generate(controller='admin/comments')
+        
+
     def test_url_with_environ_and_absolute(self):
         m = Mapper()
         m.environ = dict(SCRIPT_NAME='/blog')
@@ -505,10 +514,13 @@ class TestGeneration(unittest.TestCase):
         m.create_regs(['messages'])
         options = dict(controller='messages')
         assert '/messages' == url_for('messages')
+        assert '/messages.xml' == url_for('messages', format='xml')
         assert '/messages/1' == url_for('message', id=1)
+        assert '/messages/1.xml' == url_for('message', id=1, format='xml')
         assert '/messages/new' == url_for('new_message')
         assert '/messages/1.xml' == url_for('message', id=1, format='xml')
         assert '/messages/1;edit' == url_for('edit_message', id=1)
+        assert '/messages/1.xml;edit' == url_for('edit_message', id=1, format='xml')
         self._assert_restful_routes(m, options)
     
     def test_resources_with_path_prefix(self):
@@ -526,6 +538,8 @@ class TestGeneration(unittest.TestCase):
         self._assert_restful_routes(m, options)
         assert '/messages;rss' == m.generate(controller='messages', action='rss')
         assert '/messages;rss' == url_for('rss_messages')
+        assert '/messages.xml;rss' == m.generate(controller='messages', action='rss', format='xml')
+        assert '/messages.xml;rss' == url_for('formatted_rss_messages', format='xml')
     
     def test_resources_with_member_action(self):
         for method in ['put', 'post']:
@@ -535,6 +549,7 @@ class TestGeneration(unittest.TestCase):
             options = dict(controller='messages')
             self._assert_restful_routes(m, options)
             assert '/messages/1;mark' == m.generate(method=method, action='mark', id='1', **options)
+            assert '/messages/1.xml;mark' == m.generate(method=method, action='mark', id='1', format='xml', **options)
     
     def test_resources_with_new_action(self):
         m = Mapper()
@@ -544,6 +559,8 @@ class TestGeneration(unittest.TestCase):
         self._assert_restful_routes(m, options)
         assert '/messages/new;preview' == m.generate(controller='messages', action='preview', method='post')
         assert '/messages/new;preview' == url_for('preview_new_message')
+        assert '/messages/new.xml;preview' == m.generate(controller='messages', action='preview', method='post', format='xml')
+        assert '/messages/new.xml;preview' == url_for('preview_new_message', format='xml')
     
     def test_resources_with_name_prefix(self):
         m = Mapper()
@@ -553,8 +570,35 @@ class TestGeneration(unittest.TestCase):
         self._assert_restful_routes(m, options)
         assert '/messages/new;preview' == url_for('category_preview_new_message')
         assert None == url_for('category_preview_new_message', method='get')
+
+    def test_unicode(self):
+        hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
+        hoge_enc = urllib.quote_plus(hoge.encode('utf-8'))
+        m = Mapper()
+        m.connect(':hoge')
+        self.assertEqual("/%s" % hoge_enc, m.generate(hoge=hoge))
+        self.assertTrue(isinstance(m.generate(hoge=hoge), str))
+
+    def test_unicode_static(self):
+        hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
+        hoge_enc = urllib.quote_plus(hoge.encode('utf-8'))
+        m = Mapper()
+        m.connect('google-jp', 'http://www.google.co.jp/search', _static=True)
+        self.assertEqual("http://www.google.co.jp/search?q=" + hoge_enc,
+                         url_for('google-jp', q=hoge))
+        self.assertTrue(isinstance(url_for('google-jp', q=hoge), str))
+
+    def test_other_special_chars(self):
+        m = Mapper()
+        m.connect('/:year/:(slug).:(format),:(locale)', locale='en', format='html')
+        m.create_regs(['content'])
         
-        
+        assert '/2007/test' == m.generate(year=2007, slug='test')
+        assert '/2007/test.xml' == m.generate(year=2007, slug='test', format='xml')
+        assert '/2007/test.xml,ja' == m.generate(year=2007, slug='test', format='xml', locale='ja')
+        assert None == m.generate(year=2007, format='html')
+
+
 if __name__ == '__main__':
     unittest.main()
 else:
